# ----------------------------
# constants
# ----------------------------

FILENAME = 'movies.txt'

# --------------
# list utilities
# --------------

def list_union(list_1, list_2):
    result = []
    for item in list_1:
        result.append(item)
    for item in list_2:
        if item not in list_1:
            result.append(item)
    return result

def list_intersect(list_1, list_2):
    result = []
    for item in list_1:
        if item in list_2:
            result.append(item)
    return result

def extend_list(a_list, new_items):
    result = []
    result.extend(a_list)
    for i in new_items:
        result.append(i)
    return(result)

# ----------------------
# build_actor_database()
# ----------------------

def build_actor_database(filename=FILENAME):
    actor_db = {}

    source   = open(filename, 'r')
    for line in source:
        data_str = line.strip().split(',')
        actor  = data_str[0].strip().title()
        movies = data_str[1:]
        actor_db[actor] = []
        for movie in movies:
            movie = movie.strip().title()
            actor_db[actor].append(movie)
    source.close()
    
    return actor_db

# ----------------------
# build_movie_database()
# ----------------------

def build_movie_database(actor_db):
    movie_db = {}

    for actor in actor_db:
        movies = actor_db[actor]
        for movie in movies:
            if movie not in movie_db:
                movie_db[movie] = [actor]
            else:
                movie_db[movie].append(actor)

    return movie_db

# -------------------------------------
# acting_partners() and common_movies()
# -------------------------------------

def acting_partners(actor, actor_db, movie_db):
    if actor not in actor_db:
        return

    partners = []
    for movie in actor_db[actor]:
        actors = movie_db[movie]
        partners = list_union(partners, actors)
    partners.remove(actor)
    return partners

def common_movies(actor_1, actor_2, actor_db):
    movies_1 = actor_db[actor_1]
    movies_2 = actor_db[actor_2]
    return list_intersect(movies_1, movies_2)

# -------------------
# find_path_between()
# -------------------

def find_path_between(actor_1, actor_2):
    # load the two dbs

    actor_db = build_actor_database()
    movie_db = build_movie_database(actor_db)

    # put actor_1 in a connections db with d == 0 and p == []

    connections = {}
    connections[actor_1] = (0, [])

    level = 0
    size  = len(connections)

    # while actor_2 not in connections db
    while actor_2 not in connections:
        
        # select actors at current level
        next_actors = []
        for actor in connections:
            distance, path = connections[actor]
            if distance == level:
                next_actors.append(actor)
        
        # for each of these actors
        for actor in next_actors:
            distance, path = connections[actor]
            partners = acting_partners(actor, actor_db, movie_db)
            
            # add its partners to the connections db,
            # extending the distance and path
            for p in partners:
                if p in connections:
                    continue
                connectors = common_movies(p, actor, actor_db)
                new_path = extend_list(path, [actor, connectors])
                connections[p] = (level+1, new_path)
        
        if size == len(connections):   # there were no new links added,
            break                      # so we can stop
        level +=1
    
    if actor_2 not in connections:
        return
    return connections[actor_2]

# --------------
# display_path()
# --------------

def display_path(path, actor):
    for entry in path:
        if isinstance(entry, list):
            print('  was in', entry, 'with')
        else:
            print(entry)
    print(actor)

# -----------
# main script
# -----------

print('----------------------------------------------------')
print('            Welcome to the Movie Oracle!            ')
print('----------------------------------------------------')
print('Find the shortest path between two movie actors.' )
print('By default, the first actor is Kevin Bacon.')
print('----------------------------------------------------')

actor_1 = input('Enter first actor : ')
if not actor_1:
    actor_1 ='Kevin Bacon'
actor_2 = input('Enter second actor: ')   # 'Tom Hanks'

depth, path = find_path_between(actor_1, actor_2)

print('----------------------------------------------------')
display_path(path, actor_2)
print('----------------------------------------------------')
